<?php
define ( "BENNO_STATUS_STATE", 0 );
define ( "BENNO_STATUS_LIMIT", 1 );
define ( "BENNO_STATUS_RESULTS", 2 );

if (! class_exists ( "benno" )) {
	class benno {
		private $curl;
		private $cookiefile;
		public $letzte_suche;
		private $letzte_mail;
		public $baseurl;
		function __construct() {
			$this->init ();
		}
		function __destruct() {
			curl_close ( $this->curl );
			unlink ( $this->cookiefile );
		}
		private function init() {
			$this->curl = curl_init ();
			
			$this->cookiefile = tempnam ( sys_get_temp_dir (), "conceptERP-Cookiefile-" );
			
			curl_setopt ( $this->curl, CURLOPT_COOKIEJAR, $this->cookiefile );
			
			curl_setopt ( $this->curl, CURLOPT_RETURNTRANSFER, true );
			curl_setopt ( $this->curl, CURLOPT_AUTOREFERER, true );
			curl_setopt ( $this->curl, CURLOPT_FOLLOWLOCATION, true );
		}
		
		/*
		 * Session bei Benno erstellen
		 */
		public function login($username, $password) {
			curl_setopt ( $this->curl, CURLOPT_URL, $this->baseurl . "/bennosearch/j_spring_security_check" );
			curl_setopt ( $this->curl, CURLOPT_POSTFIELDS, "j_username=" . $username . "&j_password=" . $password );
			curl_setopt ( $this->curl, CURLOPT_POST, true );
			$return = curl_exec ( $this->curl );
			
			if (strpos ( $return, "Benutzername:" ) !== false && strpos ( $return, "Passwort:" ) !== false) {
				throw new Exception ( "Anmeldung an Benno fehlgeschlagen!" );
			}
		}
		
		/*
		 * CURL debugging
		 */
		public function debug() {
			print_r ( curl_getinfo ( $this->curl ) );
		}
		
		/*
		 * Suche absetzen
		 */
		public function search($query, $anz = 50) {
			if (! is_numeric ( $anz ))
				$anz = 50;
			
			curl_setopt ( $this->curl, CURLOPT_HTTPGET, true );
			curl_setopt ( $this->curl, CURLOPT_URL, $this->baseurl . "/bennosearch/resources/search/" . $anz . "/" . rawurlencode ( trim ( $query ) ) );
			
			$this->letzte_suche = curl_exec ( $this->curl );
			
			if (curl_getinfo ( $this->curl, CURLINFO_HTTP_CODE ) != 200 || $this->checkIsError ( $this->letzte_suche )) {
				throw new Exception ( "Fehler beim Suchen! Query: '" . $query . "' URL: '" . curl_getinfo ( $this->curl, CURLINFO_EFFECTIVE_URL ) . "'" );
			}
		}
		
		/*
		 * Prfe auf Parse Exception
		 */
		private function checkIsError($sReturn) {
			$sErrorString = "org.apache.lucene.queryParser.ParseException";
			
			if (stripos ( $sReturn, $sErrorString ) !== false) {
				return true;
			}
			
			return false;
		}
		
		/*
		 * Anzahl der Suchergebnisse
		 */
		public function getSearchResultCount() {
			if (empty ( $this->letzte_suche ))
				return 0;
			
			$status = $this->returnStatusArray ();
			
			return $status [BENNO_STATUS_RESULTS];
		}
		
		/*
		 * Anzahl der angezeigten Ergebnisse
		 */
		public function getSearchResultOutputCount() {
			if (empty ( $this->letzte_suche ))
				return 0;
			
			$status = $this->returnStatusArray ();
			
			return $status [BENNO_STATUS_LIMIT];
		}
		
		/*
		 * Array des gesamten Outputs ohne Statuszeile
		 */
		private function getSearchResultAsArray() {
			if (empty ( $this->letzte_suche ))
				return false;
			
			$array_komplett = explode ( "\n", $this->letzte_suche );
			$array_out = array ();
			
			for($i = 0; $i < count ( $array_komplett ); $i ++) {
				if ($i > 0) {
					$array_out [] = $array_komplett [$i];
				}
			}
			
			return $array_out;
		}
		
		/*
		 * Gibt ein Array zurck, welches alle Emails der Suche als
		 * BennoMailPreview Objekt enthlt
		 */
		public function getMailObjectsInArray() {
			$multiarray = array_chunk ( $this->getSearchResultAsArray (), 7 );
			$result = array ();
			
			for($i = 0; $i < count ( $multiarray ) - 1; $i ++) {
				$email = $multiarray [$i];
				$index = count ( $result );
				
				$result [$index] = new BennoMailPreview ();
				$result [$index]->Absender = $email [0];
				$result [$index]->Empfaenger = $email [1];
				$result [$index]->Status = $email [2];
				$result [$index]->Betreff = $email [3];
				$result [$index]->Text = $email [4];
				$result [$index]->DateTime = $email [5];
				$result [$index]->Id = $email [6];
			}
			
			return $result;
		}
		
		/*
		 * Gibt obiges Array zurck, ist jedoch mit Paging verwendbar
		 */
		public function getMailObjectsInArrayWithPaging($position) {
			$array_komplett = $this->getMailObjectsInArray ();
			$array_klein = array ();
			
			for($i = 0; $i < count ( $array_komplett ); $i ++) {
				if ($i < $position)
					continue;
				
				$array_klein [] = $array_komplett [$i];
			}
			
			return $array_klein;
		}
		
		/*
		 * Einzelne Mail abrufen und im original Quelltext zurckgeben
		 */
		public function get($id) {
			curl_setopt ( $this->curl, CURLOPT_HTTPGET, true );
			curl_setopt ( $this->curl, CURLOPT_URL, $this->baseurl . "/bennosearch/resources/mail/" . $id );
			return $this->letzte_mail = curl_exec ( $this->curl );
		}
		
		/*
		 * Status einer Suche als Array
		 */
		public function returnStatusArray() {
			$text = explode ( "\n", $this->letzte_suche );
			$status = explode ( " ", $text [0] );
			
			return $status;
		}
	}
}

if (! class_exists ( "BennoMailPreview" )) {
	class BennoMailPreview {
		public $Absender;
		public $Empfaenger;
		public $Status;
		public $Betreff;
		public $Text;
		public $DateTime;
		public $Id;
		public function toArray() {
			$return = array ();
			
			$return ["Absender"] = $this->Absender;
			$return ["Empfaenger"] = $this->Empfaenger;
			$return ["Status"] = $this->Status;
			$return ["Betreff"] = $this->Betreff;
			$return ["Text"] = $this->Text;
			$return ["DateTime"] = $this->DateTime;
			$return ["Id"] = $this->Id;
			
			return $return;
		}
		
		/*
		 * Zeitstempel einer Mail in ein lesbares Format bringen
		 * d.m.Y H:i
		 */
		public function getReadableDateTime() {
			$jahr = substr ( $this->DateTime, 0, 4 );
			$monat = substr ( $this->DateTime, 4, 2 );
			$tag = substr ( $this->DateTime, 6, 2 );
			$stunde = substr ( $this->DateTime, 8, 2 );
			$minute = substr ( $this->DateTime, 10, 2 );
			
			return $tag . "." . $monat . "." . $jahr . " " . $stunde . ":" . $minute;
		}
	}
}

if (! class_exists ( "BennoMailFull" )) {
	class BennoMailFull extends BennoMailPreview {
	}
}

?>