<?php

/*
 * Dateien rekursiv loeschen
 */
if (! function_exists ( "rrmdir" )) {
	function rrmdir($dir) {
		if (is_dir ( $dir )) {
			$objects = scandir ( $dir );
			foreach ( $objects as $object ) {
				if ($object != "." && $object != "..") {
					if (filetype ( $dir . "/" . $object ) == "dir")
						rrmdir ( $dir . "/" . $object );
					else
						unlink ( $dir . "/" . $object );
				}
			}
			reset ( $objects );
			rmdir ( $dir );
		}
	}
}

/*
 * Ordner kopieren
 * http://de2.php.net/manual/de/function.copy.php#91010
 */

if (! function_exists ( "kopiereOrdner" )) {
	function kopiereOrdner($src, $dst) {
		$dir = opendir ( $src );
		@mkdir ( $dst );
		while ( false !== ($file = readdir ( $dir )) ) {
			if (($file != '.') && ($file != '..')) {
				if (is_dir ( $src . '/' . $file )) {
					kopiereOrdner ( $src . '/' . $file, $dst . '/' . $file );
				} else {
					copy ( $src . '/' . $file, $dst . '/' . $file );
				}
			}
		}
		closedir ( $dir );
	}
}

/*
 * Pfad zum Content Ordner der jeweiligen Id zurueckgeben
 */

if (! function_exists ( "returnContentOrdner" )) {
	function returnContentOrdner($id) {
		global $dateipfad, $contentcontentordner;
		
		if ($id == "vorlagen") {
			$pfad = array (
					$dateipfad,
					$contentcontentordner,
					"vorlagen" 
			);
			return implode ( "/", $pfad );
		}
		
		$ebene = round ( $id / 10000 );
		if (! is_dir ( implode ( "/", array (
				$dateipfad,
				$contentcontentordner,
				$ebene 
		) ) )) {
			mkdir ( implode ( "/", array (
					$dateipfad,
					$contentcontentordner,
					$ebene 
			) ) );
		}
		
		$pfad = array (
				$dateipfad,
				$contentcontentordner,
				$ebene,
				$id 
		);
		return implode ( "/", $pfad );
	}
}

if (! function_exists ( "returnThumbnailPfad" )) {
	function returnThumbnailPfad() {
		$sPfad = sys_get_temp_dir () . "/conceptERP-Thumbnails";
		
		if (! is_dir ( $sPfad ) && is_writable ( sys_get_temp_dir () )) {
			if (mkdir ( $sPfad, 0700 )) {
				return $sPfad;
			} else {
				return false;
			}
		}
		
		return sys_get_temp_dir () . "/conceptERP-Thumbnails";
	}
}

/*
 * Zip Datei extrahieren
 */

if (! function_exists ( "extrahiereZip" )) {
	function extrahiereZip($zipdatei, $zielordner) {
		$zip = new ZipArchive ();
		if ($zip->open ( $zipdatei )) {
			$zip->extractTo ( $zielordner . "/" );
			$zip->close ();
			return true;
		} else {
			return false;
		}
	}
}

/*
 * Thumbnail erzeugen
 */
if (! function_exists ( "erzeugeThumbnail" )) {
	function erzeugeThumbnail($pfad_originalbild, $pfad_thumbnail, $width, $height, $png = false, $forceTranparency = true) {
		$aySize = getimagesize ( $pfad_originalbild );
		if (! isset ( $aySize ))
			return false;
		
		$srcWidth = $aySize [0];
		$srcHeight = $aySize [1];
		
		if ($srcWidth == 0 || $srcHeight == 0) { // Avoid div by zero
			$thWidth = 0;
			$thHeight = 0;
		} else {
			$x_magnify = $width / $srcWidth;
			$y_magnify = $height / $srcHeight;
			
			if ($x_magnify <= $y_magnify) {
				$newwidth = $srcWidth * $x_magnify;
				$newheight = $srcHeight * $x_magnify;
			} else {
				$newwidth = $srcWidth * $y_magnify;
				$newheight = $srcHeight * $y_magnify;
			}
		}
		
		switch ($aySize [2]) {
			case 1 :
				$imgPic = imagecreatefromgif ( $pfad_originalbild );
				break;
			case 2 :
				$imgPic = imagecreatefromjpeg ( $pfad_originalbild );
				break;
			case 3 :
				$imgPic = imagecreatefrompng ( $pfad_originalbild );
				break;
			default :
				return false;
		}
		
		$imgThumb = imagecreatetruecolor ( $newwidth, $newheight );
		imagecopyresampled ( $imgThumb, $imgPic, 0, 0, 0, 0, $newwidth, $newheight, $srcWidth, $srcHeight );
		
		if (! $png) {
			imagejpeg ( $imgThumb, $pfad_thumbnail );
		} else {
			if ($forceTranparency)
				setTransparency ( $imgThumb, $imgThumb );
			imagepng ( $imgThumb, $pfad_thumbnail );
		}
		
		imagedestroy ( $imgPic );
		imagedestroy ( $imgThumb );
		
		return true;
	}
}

if (! function_exists ( "setTransparency" )) {
	function setTransparency($new_image, $image_source) {
		$transparencyIndex = imagecolortransparent ( $image_source );
		$transparencyColor = array (
				'red' => 0,
				'green' => 0,
				'blue' => 0 
		);
		
		if ($transparencyIndex >= 0) {
			$transparencyColor = imagecolorsforindex ( $image_source, $transparencyIndex );
		}
		
		$transparencyIndex = imagecolorallocate ( $new_image, $transparencyColor ['red'], $transparencyColor ['green'], $transparencyColor ['blue'] );
		imagefill ( $new_image, 0, 0, $transparencyIndex );
		imagecolortransparent ( $new_image, $transparencyIndex );
	}
}

/*
 * Dateiendung ausgeben
 */

if (! function_exists ( "returnDateiendung" )) {
	function returnDateiendung($pfad, $tolower = true) {
		$dateiname = basename ( $pfad );
		
		$position_punkt = strrpos ( $dateiname, "." );
		if ($position_punkt !== false) {
			$endung = substr ( $dateiname, $position_punkt + 1 );
		} else {
			return "";
		}
		
		if ($tolower) {
			return strtolower ( $endung );
		} else {
			return $endung;
		}
	}
}

/*
 * Erzeuge Thumbnail-Pfad fr Content Bereich
 */

if (! function_exists ( "Content_returnThumbnailPfad" )) {
	function Content_returnThumbnailPfad($id, $subid, $dateiname) {
		return returnThumbnailPfad () . "/" . md5 ( $id . $subid . $dateiname ) . "." . returnDateiendung ( $dateiname );
	}
}

/*
 * Gibt die Fehlercodes der Zip Klasse zurueck
 * http://www.php.net/manual/en/function.zip-open.php#75840
 */

if (! function_exists ( "zipFileErrMsg" )) {
	function zipFileErrMsg($errno) {
		// using constant name as a string to make this function PHP4 compatible
		$zipFileFunctionsErrors = array (
				'ZIPARCHIVE::ER_MULTIDISK' => 'Multi-disk zip archives not supported.',
				'ZIPARCHIVE::ER_RENAME' => 'Renaming temporary file failed.',
				'ZIPARCHIVE::ER_CLOSE' => 'Closing zip archive failed',
				'ZIPARCHIVE::ER_SEEK' => 'Seek error',
				'ZIPARCHIVE::ER_READ' => 'Read error',
				'ZIPARCHIVE::ER_WRITE' => 'Write error',
				'ZIPARCHIVE::ER_CRC' => 'CRC error',
				'ZIPARCHIVE::ER_ZIPCLOSED' => 'Containing zip archive was closed',
				'ZIPARCHIVE::ER_NOENT' => 'No such file.',
				'ZIPARCHIVE::ER_EXISTS' => 'File already exists',
				'ZIPARCHIVE::ER_OPEN' => 'Can\'t open file',
				'ZIPARCHIVE::ER_TMPOPEN' => 'Failure to create temporary file.',
				'ZIPARCHIVE::ER_ZLIB' => 'Zlib error',
				'ZIPARCHIVE::ER_MEMORY' => 'Memory allocation failure',
				'ZIPARCHIVE::ER_CHANGED' => 'Entry has been changed',
				'ZIPARCHIVE::ER_COMPNOTSUPP' => 'Compression method not supported.',
				'ZIPARCHIVE::ER_EOF' => 'Premature EOF',
				'ZIPARCHIVE::ER_INVAL' => 'Invalid argument',
				'ZIPARCHIVE::ER_NOZIP' => 'Not a zip archive',
				'ZIPARCHIVE::ER_INTERNAL' => 'Internal error',
				'ZIPARCHIVE::ER_INCONS' => 'Zip archive inconsistent',
				'ZIPARCHIVE::ER_REMOVE' => 'Can\'t remove file',
				'ZIPARCHIVE::ER_DELETED' => 'Entry has been deleted' 
		);
		$errmsg = 'unknown';
		foreach ( $zipFileFunctionsErrors as $constName => $errorMessage ) {
			if (defined ( $constName ) and constant ( $constName ) === $errno) {
				return 'Zip File Function error: ' . $errorMessage;
			}
		}
		return 'Zip File Function error: unknown';
	}
}

/*
 * Bytes in am besten geeignete Einheit wandeln
 * http://de2.php.net/manual/de/function.filesize.php#100097
 */

if (! function_exists ( "format_bytes" )) {
	function format_bytes($size) {
		$units = array (
				' B',
				' KB',
				' MB',
				' GB',
				' TB' 
		);
		for($i = 0; $size >= 1024 && $i < 4; $i ++)
			$size /= 1024;
		return double_from_sql ( $size, 2 ) . $units [$i];
	}
}

/*
 * Ordnerstruktur -> XML
 */
define ( "GET_FS_XML_MODE_ALL", 1 );
define ( "GET_FS_XML_MODE_DIRS", 2 );
define ( "GET_FS_XML_MODE_FILES", 3 );

if (! function_exists ( "get_filesystem_xml" )) {
	function get_filesystem_xml($dir, SimpleXMLElement $parent, $mode = GET_FS_XML_MODE_ALL) {
		$it = new DirectoryIterator ( $dir );
		
		while ( $it->valid () ) {
			if ($it->isFile () && $mode != GET_FS_XML_MODE_DIRS) {
				$child = $parent->addChild ( 'file' );
				$child->addAttribute ( 'name', utf8_encode ( $it->getFilename () ) );
				$child->addAttribute ( 'formattedsize', format_bytes ( $it->getSize () ) );
				$child->addAttribute ( 'bytesize', $it->getSize () );
				$child->addAttribute ( 'mtime', date ( "d.m.Y H:i", $it->getMTime () ) );
			} elseif ($it->isDir () && ! $it->isDot () && $mode != GET_FS_XML_MODE_FILES) {
				$child = $parent->addChild ( 'folder' );
				$child->addAttribute ( 'name', $it->getFilename () );
				get_filesystem_xml ( $it->getPathname (), $child, $mode );
			}
			
			$it->next ();
		}
	}
}

/*
 * Suche in einem Verzeichnis rekursiv nach Dateien und
 * schreibe deren Pfade in ein Array
 */
if (! function_exists ( "get_all_files_in_array" )) {
	function get_all_files_in_array($dir, &$outputArray) {
		$it = new DirectoryIterator ( $dir );
		
		while ( $it->valid () ) {
			if ($it->isFile ()) {
				$outputArray [] = $it->getPathname ();
			} elseif ($it->isDir () && ! $it->isDot ()) {
				get_all_files_in_array ( $it->getPathname (), $outputArray );
			}
			
			$it->next ();
		}
	}
}

/*
 * Mehrere PDF zu einer machen (Seiten anhngen)
 */
if (! function_exists ( "attachPdfToPdf" )) {
	function attachPdfToPdf($files, $output) {
		global $pfad_pdftf;
		
		$filesText = implode ( " ", ( array ) $files );
		
		if (! is_writable ( dirname ( $output ) )) {
			return false;
		}
		
		exec ( $pfad_pdftf . " " . $filesText . " cat output " . $output );
	}
}

/*
 * Dateien mithilfe von Wildcards lschen
 */
if (! function_exists ( "file_delete_wildcard" )) {
	function file_delete_wildcard($filestring) {
		foreach ( glob ( $filestring ) as $filename ) {
			unlink ( $filename );
		}
	}
}

/*
 * Liest ein modul.info File aus und gibt die Informationen in einem Array zurck
 */
if (! function_exists ( "parseModulInfo" )) {
	function parseModulInfo($sPath) {
		$aInfo = array ();
		
		if (is_readable ( $sPath )) {
			$sFileContents = file_get_contents ( $sPath );
			$aFileLines = explode ( "\n", $sFileContents );
			
			foreach ( $aFileLines as $sLine ) {
				$sLine = trim ( $sLine );
				$aPieces = explode ( ":", $sLine, 2 );
				
				if (count ( $aPieces ) == 2) {
					$aInfo [strtolower ( $aPieces [0] )] = $aPieces [1];
				}
			}
		}
		
		return $aInfo;
	}
}

/*
 * Formatiere Filename - lsche alles bis auf erlaubte Zeichen
 */
if (! function_exists ( format_filename )) {
	function format_filename($value) {
		// ersetze Umlaute
		$value = str_replace ( array (
				"",
				"",
				"",
				"",
				"",
				"",
				"",
				" ",
				"<br>" 
		), array (
				"Ae",
				"Oe",
				"Ue",
				"ae",
				"oe",
				"ue",
				"ss",
				"_",
				"_" 
		), $value );
		
		// lsche alle nicht erlaubten Zeichen
		$value = format_string_allowed_chars ( $value, array (
				"A",
				"B",
				"C",
				"D",
				"E",
				"F",
				"G",
				"H",
				"I",
				"J",
				"K",
				"L",
				"M",
				"N",
				"O",
				"P",
				"Q",
				"R",
				"S",
				"T",
				"U",
				"V",
				"W",
				"X",
				"Y",
				"Z",
				"a",
				"b",
				"c",
				"d",
				"e",
				"f",
				"g",
				"h",
				"i",
				"j",
				"k",
				"l",
				"m",
				"n",
				"o",
				"p",
				"q",
				"r",
				"s",
				"t",
				"u",
				"v",
				"w",
				"x",
				"y",
				"z",
				"0",
				"1",
				"2",
				"3",
				"4",
				"5",
				"6",
				"7",
				"8",
				"9",
				"-",
				"_" 
		) );
		
		// entferne Leerzeichen am Anfang
		while ( substr ( $value, 0, 1 ) == "_" ) {
			$value = substr ( $value, 1 );
		}
		
		// entferne Leerzeichen am Ende
		while ( substr ( $value, - 1 ) == "_" ) {
			$value = substr ( $value, 0, - 1 );
		}
		
		return $value;
	}
}

?>